//+------------------------------------------------------------------+
//|                                                   T3 crosses.mq4 |
//|                                                           mladen |
//|                                                                  |
//|                                                                  |
//| original T3 developed by Tim Tilson (TASC January 1998)          |
//| period lag modification by Bob Fulks and Alex Matulich 4/2003    |
//+------------------------------------------------------------------+

#property copyright "copyleft mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers 6
#property indicator_color1 Red
#property indicator_color2 Maroon
#property indicator_color3 LimeGreen
#property indicator_color4 Green
#property indicator_color5 Gold
#property indicator_color6 Green
#property indicator_width1 2
#property indicator_width2 1
#property indicator_width3 2
#property indicator_width4 1
#property indicator_width5 2
#property indicator_width6 2

//
//
//
//
//

extern string TimeFrame      = "Current time frame";
extern int    T3FastPeriod   = 20;
extern int    T3FastPrice    = PRICE_CLOSE;
extern double T3FastHot      = 1.2;
extern bool   T3FastOriginal = false;
extern int    T3SlowPeriod   = 35;
extern int    T3SlowPrice    = PRICE_CLOSE;
extern double T3SlowHot      = 1.2;
extern bool   T3SlowOriginal = false;
extern bool   Interpolate    = true;

//
//
//
//
//

double t3gCross[];
double t3rCross[];
double upBuffer1[];
double upBuffer2[];
double dnBuffer1[];
double dnBuffer2[];
double t3diff[];
double trend[];
double emas[][12];
double alphaFast,alphaSlow;
double c1fast,c1slow;
double c2fast,c2slow;
double c3fast,c3slow;
double c4fast,c4slow;

//
//
//
//
//

string indicatorFileName;
bool   calculateValue;
bool   returnBars;
int    timeFrame;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(8);
      SetIndexBuffer(0,dnBuffer1); SetIndexStyle(0,DRAW_HISTOGRAM);
      SetIndexBuffer(1,dnBuffer2); SetIndexStyle(1,DRAW_HISTOGRAM);
      SetIndexBuffer(2,upBuffer1); SetIndexStyle(2,DRAW_HISTOGRAM);
      SetIndexBuffer(3,upBuffer2); SetIndexStyle(3,DRAW_HISTOGRAM);
      SetIndexBuffer(4,t3rCross);  
      SetIndexBuffer(5,t3gCross);   
      SetIndexBuffer(6,t3diff);  
      SetIndexBuffer(7,trend);
      
      SetIndexLabel(4,"T3 trix "+T3SlowPeriod);
      SetIndexLabel(5,"T3 trix "+T3FastPeriod);   

   //
   //
   //
   //
   //
   
      T3FastPeriod = MathMax(1,T3FastPeriod);
      if (T3FastOriginal)
           alphaFast = 2.0/(1.0 + T3FastPeriod);
      else alphaFast = 2.0/(2.0 + (T3FastPeriod-1.0)/2.0);

      double a  = T3FastHot;
             c1fast = -a*a*a;
             c2fast = 3*a*a+3*a*a*a;
             c3fast = -6*a*a-3*a-3*a*a*a;
             c4fast = 1+3*a+a*a*a+3*a*a;


      //
      //
      //
      //
      //
      
      T3SlowPeriod = MathMax(1,T3SlowPeriod);
      if (T3SlowOriginal)
           alphaSlow = 2.0/(1.0 + T3SlowPeriod);
      else alphaSlow = 2.0/(2.0 + (T3SlowPeriod-1.0)/2.0);

             a  = T3SlowHot;
             c1slow = -a*a*a;
             c2slow = 3*a*a+3*a*a*a;
             c3slow = -6*a*a-3*a-3*a*a*a;
             c4slow = 1+3*a+a*a*a+3*a*a;

   
      //
      //
      //
      //
      //
      
         indicatorFileName = WindowExpertName();
         calculateValue    = TimeFrame=="calculateValue"; if (calculateValue) { return(0); }
         returnBars        = TimeFrame=="returnBars";     if (returnBars)     { return(0); }
         timeFrame         = stringToTimeFrame(TimeFrame);
         
   IndicatorShortName(timeFrameToString(timeFrame)+" T3 trix ("+T3FastPeriod+", "+DoubleToStr(T3FastHot,2)+"; "+T3SlowPeriod+", "+DoubleToStr(T3SlowHot,2)+"");
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double work[][2];

int start()
{
   int i,r,limit,counted_bars=IndicatorCounted();

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
           limit=MathMin(Bars-counted_bars,Bars-1);
           if (returnBars) { dnBuffer1[0] = MathMin(limit+1,Bars-1); return(0); }

   //
   //
   //
   //
   //


   if (calculateValue || timeFrame==Period())
   {
      if (ArrayRange(emas,0) != Bars) { ArrayResize(emas,Bars); ArrayResize(work,Bars); }
      for(i=limit, r=Bars-i-1; i>=0; i--,r++)
      {
       
         work[r][0]   = iT3(iMA(NULL,0,1,0,MODE_SMA,T3FastPrice,i),i,0,alphaFast,c1fast,c2fast,c3fast,c4fast);  
         work[r][1]   = iT3(iMA(NULL,0,1,0,MODE_SMA,T3SlowPrice,i),i,6,alphaSlow,c1slow,c2slow,c3slow,c4slow);
         t3gCross[i]  = work[r][0]-work[r-1][0];
         t3rCross[i]  = work[r][1]-work[r-1][1];
         t3diff[i]    = t3gCross[i]-t3rCross[i];
         upBuffer1[i] = EMPTY_VALUE;
         upBuffer2[i] = EMPTY_VALUE;
         dnBuffer1[i] = EMPTY_VALUE;
         dnBuffer2[i] = EMPTY_VALUE;
         trend[i]     = trend[i+1];
            if (t3diff[i]>t3diff[i+1]) trend[i] =  1;
            if (t3diff[i]<t3diff[i+1]) trend[i] = -1;
            if (t3diff[i]>0)
               if (trend[i] == 1)
                     upBuffer1[i] = t3diff[i];
               else  upBuffer2[i] = t3diff[i];
            else               
               if (trend[i] == 1)
                     dnBuffer2[i] = t3diff[i];
               else  dnBuffer1[i] = t3diff[i];
      }
      return(0);
   }
   
   //
   //
   //
   //
   //

   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for (i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         t3gCross[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",T3FastPeriod,T3FastPrice,T3FastHot,T3FastOriginal,T3SlowPeriod,T3SlowPrice,T3SlowHot,T3SlowOriginal,4,y);
         t3rCross[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",T3FastPeriod,T3FastPrice,T3FastHot,T3FastOriginal,T3SlowPeriod,T3SlowPrice,T3SlowHot,T3SlowOriginal,5,y);
         t3diff[i]   = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",T3FastPeriod,T3FastPrice,T3FastHot,T3FastOriginal,T3SlowPeriod,T3SlowPrice,T3SlowHot,T3SlowOriginal,6,y);
         trend[i]    = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",T3FastPeriod,T3FastPrice,T3FastHot,T3FastOriginal,T3SlowPeriod,T3SlowPrice,T3SlowHot,T3SlowOriginal,7,y);
         upBuffer1[i] = EMPTY_VALUE;
         upBuffer2[i] = EMPTY_VALUE;
         dnBuffer1[i] = EMPTY_VALUE;
         dnBuffer2[i] = EMPTY_VALUE;
         if (t3diff[i]>0)
            if (trend[i] == 1)
                  upBuffer1[i] = t3diff[i];
            else  upBuffer2[i] = t3diff[i];
         else               
            if (trend[i] == 1)
                  dnBuffer2[i] = t3diff[i];
            else  dnBuffer1[i] = t3diff[i];

         //
         //
         //
         //
         //

         if (!Interpolate || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;

         //
         //
         //
         //
         //

         datetime time = iTime(NULL,timeFrame,y);
            for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
            for(int k = 1; k < n; k++)
            {
               t3gCross[i+k] = t3gCross[i] + (t3gCross[i+n]-t3gCross[i])*k/n;
               t3rCross[i+k] = t3rCross[i] + (t3rCross[i+n]-t3rCross[i])*k/n;
               t3diff[i+k]   = t3diff[i]   + (t3diff[i+n]  -t3diff[i])*k/n;
               
                  //
                  //
                  //
                  //
                  //
                  
                  if (upBuffer1[i+k]!=EMPTY_VALUE) upBuffer1[i+k] = t3diff[i+k];
                  if (upBuffer2[i+k]!=EMPTY_VALUE) upBuffer2[i+k] = t3diff[i+k];
                  if (dnBuffer1[i+k]!=EMPTY_VALUE) dnBuffer1[i+k] = t3diff[i+k];
                  if (dnBuffer2[i+k]!=EMPTY_VALUE) dnBuffer2[i+k] = t3diff[i+k];
            }
                  
   }

   //
   //
   //
   //
   //
   
   return(0);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double iT3(double price,int shift,int startBuff,double alpha,double c1,double c2,double c3,double c4)
{
   int i = Bars-shift-1;
   if (i < 1)
      {
         emas[i][startBuff+0] = price;
         emas[i][startBuff+1] = price;
         emas[i][startBuff+2] = price;
         emas[i][startBuff+3] = price;
         emas[i][startBuff+4] = price;
         emas[i][startBuff+5] = price;
      }
   else
      {
         emas[i][startBuff+0] = emas[i-1][startBuff+0]+alpha*(price               -emas[i-1][startBuff+0]);
         emas[i][startBuff+1] = emas[i-1][startBuff+1]+alpha*(emas[i][startBuff+0]-emas[i-1][startBuff+1]);
         emas[i][startBuff+2] = emas[i-1][startBuff+2]+alpha*(emas[i][startBuff+1]-emas[i-1][startBuff+2]);
         emas[i][startBuff+3] = emas[i-1][startBuff+3]+alpha*(emas[i][startBuff+2]-emas[i-1][startBuff+3]);
         emas[i][startBuff+4] = emas[i-1][startBuff+4]+alpha*(emas[i][startBuff+3]-emas[i-1][startBuff+4]);
         emas[i][startBuff+5] = emas[i-1][startBuff+5]+alpha*(emas[i][startBuff+4]-emas[i-1][startBuff+5]);
      }
   return(c1*emas[i][startBuff+5] + c2*emas[i][startBuff+4] + c3*emas[i][startBuff+3] + c4*emas[i][startBuff+2]);
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}